@@ -3,17 +3,18 @@
  *      Guido Draheim <guidod@gmx.de>
  *      Tomi Ollila <too@iki.fi>
  *
- *      Copyright (c) 1999,2000,2001,2002 Guido Draheim
+ * Copyright (c) 1999,2000,2001,2002,2003 Guido Draheim
  *          All rights reserved,
  *          use under the restrictions of the 
  *          Lesser GNU General Public License
- *          note the additional license information 
- *          that can be found in COPYING.ZZIP
+ *          or alternatively the restrictions 
+ *          of the Mozilla Public License 1.1
  */
 
 #include <zzip/lib.h>                                  /* archive handling */
 #include <zzip/file.h>
 #include <zzip/format.h>
+#include <zzip/fetch.h>
 
 #include <ctype.h>
 #include <errno.h>
@@ -32,25 +33,14 @@
 /* per default, we use a little hack to correct bad z_rootseek parts */
 #define ZZIP_CORRECT_ROOTSEEK 1
 
-/* ------------------------- fetch helpers --------------------------------- */
-
-/**
- * Make 32 bit value in host byteorder from little-endian mapped octet-data
- * (works also on machines which SIGBUS on misaligned data access (eg. 68000))
- */
-uint32_t __zzip_get32(unsigned char * s)
-{
-  return ((uint32_t)s[3] << 24) | ((uint32_t)s[2] << 16)
-    |    ((uint32_t)s[1] << 8)  |  (uint32_t)s[0];
-}
-
-/** => __zzip_get32
- * This function does the same for a 16 bit value.
- */
-uint16_t __zzip_get16(unsigned char * s)
-{
-    return ((uint16_t)s[1] << 8) | (uint16_t)s[0];
-}
+#if __GNUC__ >= 3 && __GNUC_MINOR__ >= 3
+# ifdef DEBUG
+# warning suppress a warning where the compiler should have optimized instead.
+# endif
+#define _255 254
+#else
+#define _255 255
+#endif
 
 /* ---------------------------  internals  -------------------------------- */
 /* internal functions of zziplib, avoid at all cost, changes w/o warning.
@@ -80,18 +70,14 @@ _zzip_inline static void __fixup_rootseek(
     zzip_off_t offset_of_trailer,
     struct zzip_disk_trailer* trailer)
 {
-    if (                    (zzip_off_t) ZZIP_GET32(trailer->z_rootseek) >
-	offset_of_trailer - (zzip_off_t) ZZIP_GET32(trailer->z_rootsize) &&
-	offset_of_trailer > (zzip_off_t) ZZIP_GET32(trailer->z_rootsize))
+    if (                    zzip_disk_trailer_rootseek (trailer) >
+	offset_of_trailer - zzip_disk_trailer_rootsize (trailer) &&
+	offset_of_trailer > zzip_disk_trailer_rootsize (trailer))
     {
 	register zzip_off_t offset;
-	offset = offset_of_trailer -  ZZIP_GET32(trailer->z_rootsize); 
-	trailer->z_rootseek[0] = offset & 0xff;
-	trailer->z_rootseek[1] = offset >> 8 & 0xff;
-	trailer->z_rootseek[2] = offset >> 16 & 0xff;
-	trailer->z_rootseek[3] = offset >> 24 & 0xff;
-	HINT2("new rootseek=%li", 
-	      (long) ZZIP_GET32(trailer->z_rootseek));
+	offset = offset_of_trailer - zzip_disk_trailer_rootsize (trailer);
+	zzip_disk_trailer_set_rootseek (trailer, offset);
+	HINT2("new rootseek=%li", (long) zzip_disk_trailer_rootseek (trailer));
     }
 }
 #define __correct_rootseek(A,B,C)
@@ -115,7 +101,7 @@ _zzip_inline static void __fixup_rootseek(
 #ifdef DEBUG
 _zzip_inline static void __debug_dir_hdr (struct zzip_dir_hdr* hdr)
 {
-    if (sizeof(struct zzip_dir_hdr) > sizeof(struct zzip_root_dirent))
+    if (sizeof(struct zzip_dir_hdr) > sizeof(struct zzip_disk_entry))
     { WARN1("internal sizeof-mismatch may break wreakage"); }
     /*  the internal directory structure is never bigger than the
      *  external zip central directory space had been beforehand
@@ -171,7 +157,7 @@ __zzip_find_disk_trailer(int fd, zzip_off_t filesize,
     char* buf = malloc(2*ZZIP_BUFSIZ);
 #endif
     zzip_off_t offset = 0;
-    zzip_off_t maplen = 0; /* mmap(),read(),getpagesize() use size_t !! */
+    zzip_ssize_t maplen = 0; /* mmap(),read(),getpagesize() use size_t !! */
     char* fd_map = 0;
 
     if (!trailer)
@@ -188,27 +174,27 @@ __zzip_find_disk_trailer(int fd, zzip_off_t filesize,
     {
         register unsigned char* mapped;
 
-         if (offset <= 0) { return(ZZIP_DIR_EDH_MISSING); }
+	 if (offset <= 0) { return(ZZIP_DIR_EDH_MISSING); }
 
 	 /* trailer cannot be farther away than 64K from fileend */
          if (filesize-offset > 64*1024) 
              { return(ZZIP_DIR_EDH_MISSING); }
 
 	/* the new offset shall overlap with the area after the old offset! */
-        if (USE_MMAP && io->use_mmap)
+        if (USE_MMAP && io->fd.sys)
         {
 	    zzip_off_t mapoff = offset;
 	    { 
-		zzip_off_t pagesize = _zzip_getpagesize (io->use_mmap);
+		zzip_ssize_t pagesize = _zzip_getpagesize (io->fd.sys);
 		if (pagesize < ZZIP_BUFSIZ) goto non_mmap; /* an error? */
 		if (mapoff == filesize && filesize > pagesize) 
 		    mapoff -= pagesize;
 		if (mapoff < pagesize) {
-		    maplen = mapoff + pagesize; mapoff = 0;
+		    maplen = (zzip_ssize_t)mapoff + pagesize; mapoff = 0;
 		} else {               
 		    mapoff -= pagesize; maplen = 2*pagesize; 
-		    if (mapoff & (pagesize-1)) { /* only on first run */
-			pagesize -= mapoff & (pagesize-1);
+		    if ((zzip_ssize_t)mapoff & (pagesize-1)) { /*only 1. run */
+			pagesize -= (zzip_ssize_t)mapoff & (pagesize-1);
 			mapoff += pagesize;
 			maplen -= pagesize;
 		    }   
@@ -216,9 +202,9 @@ __zzip_find_disk_trailer(int fd, zzip_off_t filesize,
 		if (mapoff + maplen > filesize) maplen = filesize - mapoff;
 	    }
 
-            fd_map = _zzip_mmap(io->use_mmap, fd, mapoff, (zzip_size_t)maplen);
+            fd_map = _zzip_mmap(io->fd.sys, fd, mapoff, (zzip_size_t)maplen);
             if (fd_map == MAP_FAILED) goto non_mmap;
-	    mapped = fd_map; offset = mapoff; /* success */
+	    mapped = (unsigned char*) fd_map; offset = mapoff; /* success */
 	    HINT3("mapped *%p len=%li", fd_map, (long) maplen);
         } else {
         non_mmap:
@@ -228,11 +214,11 @@ __zzip_find_disk_trailer(int fd, zzip_off_t filesize,
 		if (offset == filesize && filesize > pagesize)
 		    offset -= pagesize;
 		if (offset < pagesize) {
-		    maplen = offset + pagesize; offset = 0;
+		    maplen = (zzip_ssize_t)offset + pagesize; offset = 0;
 		} else {
 		    offset -= pagesize; maplen = 2*pagesize;
-		    if (offset & (pagesize-1)) { /* only on first run */
-			pagesize -= offset & (pagesize-1);
+		    if ((zzip_ssize_t)offset & (pagesize-1)) { /*on 1st run*/
+			pagesize -= (zzip_ssize_t)offset & (pagesize-1);
 			offset += pagesize;
 			maplen -= pagesize; 
 		    }    
@@ -240,11 +226,11 @@ __zzip_find_disk_trailer(int fd, zzip_off_t filesize,
 		if (offset + maplen > filesize) maplen = filesize - offset;
 	    }
 	    
-            if (io->seeks(fd, offset, SEEK_SET) < 0)
+            if (io->fd.seeks(fd, offset, SEEK_SET) < 0)
                 { return(ZZIP_DIR_SEEK); }
-            if (io->read(fd, buf, (zzip_size_t)maplen) < (zzip_ssize_t)maplen)
+            if (io->fd.read(fd, buf, (zzip_size_t)maplen) < maplen)
                 { return(ZZIP_DIR_READ); }
-            mapped = buf; /* success */
+            mapped = (unsigned char*) buf; /* success */
 	    HINT5("offs=$%lx len=%li filesize=%li pagesize=%i", 
 		  (long)offset, (long)maplen, (long)filesize, ZZIP_BUFSIZ);
         }
@@ -278,7 +264,8 @@ __zzip_find_disk_trailer(int fd, zzip_off_t filesize,
          if (USE_MMAP && fd_map) 
 	 { 
 	     HINT3("unmap *%p len=%li",  fd_map, (long) maplen);
-	     _zzip_munmap(io->use_mmap, fd_map, (zzip_size_t)maplen); fd_map = 0; 
+	     _zzip_munmap(io->fd.sys, fd_map, (zzip_size_t)maplen); 
+	     fd_map = 0; 
 	 }
     } /*outer loop*/
                
@@ -286,7 +273,7 @@ __zzip_find_disk_trailer(int fd, zzip_off_t filesize,
     if (USE_MMAP && fd_map)
     { 
 	HINT3("unmap *%p len=%li",  fd_map, (long) maplen);
-	_zzip_munmap(io->use_mmap, fd_map, (zzip_size_t)maplen); 
+	_zzip_munmap(io->fd.sys, fd_map, (zzip_size_t)maplen); 
     }
 #   ifdef _LOWSTK
     free(buf);
@@ -322,7 +309,7 @@ __zzip_parse_root_directory(int fd,
     struct zzip_dir_hdr ** hdr_return,
     zzip_plugin_io_t io)
 {
-    auto struct zzip_root_dirent dirent;
+    auto struct zzip_disk_entry dirent;
     struct zzip_dir_hdr * hdr;
     struct zzip_dir_hdr * hdr0;
     uint16_t * p_reclen = 0;
@@ -330,9 +317,9 @@ __zzip_parse_root_directory(int fd,
     long offset;          /* offset from start of root directory */
     char* fd_map = 0; 
     int32_t  fd_gap = 0;
-    uint16_t u_entries  = ZZIP_GET16(trailer->z_entries);   
-    uint32_t u_rootsize = ZZIP_GET32(trailer->z_rootsize);  
-    uint32_t u_rootseek = ZZIP_GET32(trailer->z_rootseek);
+    uint16_t u_entries  = zzip_disk_trailer_get_entries (trailer);   
+    uint32_t u_rootsize = zzip_disk_trailer_get_rootsize (trailer);  
+    uint32_t u_rootseek = zzip_disk_trailer_get_rootseek (trailer);
     __correct_rootseek (u_rootseek, u_rootsize, trailer);
 
     hdr0 = (struct zzip_dir_hdr*) malloc(u_rootsize);
@@ -340,13 +327,13 @@ __zzip_parse_root_directory(int fd,
         return ZZIP_DIRSIZE;
     hdr = hdr0;                  __debug_dir_hdr (hdr);
 
-    if (USE_MMAP && io->use_mmap)
+    if (USE_MMAP && io->fd.sys)
     {
-        fd_gap = u_rootseek & (_zzip_getpagesize(io->use_mmap)-1) ;
+        fd_gap = u_rootseek & (_zzip_getpagesize(io->fd.sys)-1) ;
         HINT4(" mapseek=0x%x, maplen=%d, fd_gap=%d", 
 	      u_rootseek-fd_gap, u_rootsize+fd_gap, fd_gap);
-        fd_map = _zzip_mmap(io->use_mmap, 
-			    fd, u_rootseek-fd_gap, u_rootsize+fd_gap);
+        fd_map = _zzip_mmap(io->fd.sys, fd,
+			    u_rootseek-fd_gap, u_rootsize+fd_gap);
         /* if mmap failed we will fallback to seek/read mode */
         if (fd_map == MAP_FAILED) { 
             NOTE2("map failed: %s",strerror(errno)); 
@@ -358,16 +345,16 @@ __zzip_parse_root_directory(int fd,
 
     for (entries=u_entries, offset=0; entries > 0; entries--)
     {
-        register struct zzip_root_dirent * d;
+        register struct zzip_disk_entry * d;
         uint16_t u_extras, u_comment, u_namlen;
 
         if (fd_map) 
 	{ d = (void*)(fd_map+fd_gap+offset); } /* fd_map+fd_gap==u_rootseek */
         else
         {
-            if (io->seeks(fd, u_rootseek+offset, SEEK_SET) < 0)
+            if (io->fd.seeks(fd, u_rootseek+offset, SEEK_SET) < 0)
                 return ZZIP_DIR_SEEK;
-            if (io->read(fd, &dirent, sizeof(dirent)) < __sizeof(dirent))
+            if (io->fd.read(fd, &dirent, sizeof(dirent)) < __sizeof(dirent))
                 return ZZIP_DIR_READ;
             d = &dirent;
         }
@@ -379,9 +366,9 @@ __zzip_parse_root_directory(int fd,
         zzip_debug_xbuf ((unsigned char*) d, sizeof(*d) + 8);
 #       endif        
         
-        u_extras  = ZZIP_GET16(d->z_extras); 
-        u_comment = ZZIP_GET16(d->z_comment); 
-        u_namlen  = ZZIP_GET16(d->z_namlen); 
+        u_extras  = zzip_disk_entry_get_extras (d);
+        u_comment = zzip_disk_entry_get_comment (d); 
+        u_namlen  = zzip_disk_entry_get_namlen (d); 
         HINT5("offset=0x%lx, size %ld, dirent *%p, hdr %p\n",
 	      offset+u_rootseek, (long)u_rootsize, d, hdr);
 
@@ -392,19 +379,19 @@ __zzip_parse_root_directory(int fd,
            first structure read.
            at the end the whole copied list of structures  is copied into
            newly allocated buffer */
-        hdr->d_crc32 = ZZIP_GET32(d->z_crc32);
-        hdr->d_csize = ZZIP_GET32(d->z_csize); 
-        hdr->d_usize = ZZIP_GET32(d->z_usize); 
-        hdr->d_off   = ZZIP_GET32(d->z_off);
-        hdr->d_compr = (uint8_t)ZZIP_GET16(d->z_compr);
-        if (hdr->d_compr > 255) hdr->d_compr = 255;
+        hdr->d_crc32 = zzip_disk_entry_get_crc32 (d);
+        hdr->d_csize = zzip_disk_entry_get_csize (d); 
+        hdr->d_usize = zzip_disk_entry_get_usize (d);
+        hdr->d_off   = zzip_disk_entry_get_offset (d);
+        hdr->d_compr = zzip_disk_entry_get_compr (d);
+	if (hdr->d_compr > _255) hdr->d_compr = 255;
 
 	if (offset+sizeof(*d) + u_namlen > u_rootsize)
 	{ FAIL2("%i's name stretches beyond root directory", entries); break;}
 
 	if (fd_map) 
-	{  memcpy(hdr->d_name, fd_map+fd_gap+offset+sizeof(*d), u_namlen); }
-	else { io->read(fd, hdr->d_name, u_namlen); }
+	{  memcpy(hdr->d_name, fd_map+fd_gap + offset+sizeof(*d), u_namlen); }
+	else { io->fd.read(fd, hdr->d_name, u_namlen); }
         hdr->d_name[u_namlen] = '\0'; 
         hdr->d_namlen = u_namlen;
     
@@ -433,7 +420,7 @@ __zzip_parse_root_directory(int fd,
     if (USE_MMAP && fd_map) 
     {
 	HINT3("unmap *%p len=%i",   fd_map, u_rootsize+fd_gap);
-        _zzip_munmap(io->use_mmap, fd_map, u_rootsize+fd_gap);
+        _zzip_munmap(io->fd.sys, fd_map, u_rootsize+fd_gap);
     }
     
     if (p_reclen)
@@ -516,7 +503,7 @@ zzip_dir_free(ZZIP_DIR * dir)
     if (dir->refcount)
         return (dir->refcount); /* still open files attached */
 
-    if (dir->fd >= 0)      dir->io->close(dir->fd);
+    if (dir->fd >= 0)      dir->io->fd.close(dir->fd);
     if (dir->hdr0)         free(dir->hdr0);
     if (dir->cache.fp)     free(dir->cache.fp);
     if (dir->cache.buf32k) free(dir->cache.buf32k);
@@ -525,7 +512,7 @@ zzip_dir_free(ZZIP_DIR * dir)
     return 0;
 }
 
-/**
+/** => zzip_dir_free
  * It will also => free(2) the => ZZIP_DIR-handle given. 
  * the counterpart for => zzip_dir_open
  * see also => zzip_dir_free
@@ -542,7 +529,7 @@ zzip_dir_close(ZZIP_DIR * dir)
  * zip-archive as specified with the fd which points to an
  * already openend file. This function then search and parse
  * the zip's central directory.
- * <p> 
+ *  
  * NOTE: refcount is zero, so an _open/_close pair will also delete 
  *       this _dirhandle 
  */
@@ -594,7 +581,7 @@ __zzip_dir_parse (ZZIP_DIR* dir)
      */
 
     HINT2("------------------ fd=%i", (int) dir->fd);
-    if ((filesize = dir->io->filesize(dir->fd)) < 0)
+    if ((filesize = dir->io->fd.filesize(dir->fd)) < 0)
         { rv = ZZIP_DIR_STAT; goto error; }
 
     HINT2("------------------ filesize=%ld", (long) filesize);
@@ -602,9 +589,11 @@ __zzip_dir_parse (ZZIP_DIR* dir)
                                        dir->io)) != 0)
         { goto error; }
                 
-    HINT5("directory = { entries= %d/%d, size= %d, seek= %d } ", 
-	  ZZIP_GET16(trailer.z_entries),  ZZIP_GET16(trailer.z_finalentries),
-	  ZZIP_GET32(trailer.z_rootsize), ZZIP_GET32(trailer.z_rootseek));
+    HINT5("directory = { entries= %ld/%ld, size= %ld, seek= %ld } ", 
+	  (long) zzip_disk_trailer_localentries (&trailer),
+	  (long) zzip_disk_trailer_finalentries (&trailer),
+	  (long) zzip_disk_trailer_rootsize (&trailer),
+	  (long) zzip_disk_trailer_rootseek (&trailer));
     
     if ( (rv = __zzip_parse_root_directory(dir->fd, &trailer, &dir->hdr0, 
                                            dir->io)) != 0)
@@ -635,7 +624,7 @@ __zzip_try_open(zzip_char_t* filename, int filemode,
     for ( ; *ext ; ++ext)
     {
         strcpy (file+len, *ext);
-        fd = io->open(file, filemode);
+        fd = io->fd.open(file, filemode);
         if (fd != -1) return fd;
     }
     return -1;
@@ -665,7 +654,7 @@ zzip_dir_open_ext_io(zzip_char_t* filename, zzip_error_t* e,
     if (!io) io = zzip_get_default_io();
     if (!ext) ext = zzip_get_default_ext();
 
-    fd = io->open(filename, O_RDONLY|O_BINARY);
+    fd = io->fd.open(filename, O_RDONLY|O_BINARY);
     if (fd != -1) 
       { return zzip_dir_fdopen_ext_io(fd, e, ext, io); }
     else
@@ -681,10 +670,10 @@ zzip_dir_open_ext_io(zzip_char_t* filename, zzip_error_t* e,
     }
 }
 
-/**
+/** => zzip_dir_open
  * fills the dirent-argument with the values and 
  * increments the read-pointer of the dir-argument.
- * <p>
+ * 
  * returns 0 if there no entry (anymore).
  */
 int
